/*
    Script to create a maintenance partition or a bootable CD

    Written by Hayo Baan (hrbaan@xs4all.nl)
    http://www.xs4all.nl/~hrbaan

    Please see booAble.html for instructions and version history

    Version 2.17 (19/12/2003)
*/

'@echo off'

/* Make sure environment is local */
rc=setLocal()

/* Load the rexxUtil functions                                              */
call RxFuncAdd 'SysLoadFuncs', 'RexxUtil', 'SysLoadFuncs'
call SysLoadFuncs

/* String variable to insert a newline in a character string                */
CRLF=d2c(13)||d2c(10)

/* Read the configuration file(s)                                           */
rc=readConfig(ARG(1))

/* Determine if command output should be shown                              */
select
    when VERBOSE=9 then IGNOREOUTPUT=''
    when VERBOSE=3 then IGNOREOUTPUT='> /dev/nul'
    otherwise IGNOREOUTPUT='> /dev/nul 2>&1'
end

/* Create temp directory                                                    */
TEMPDIR=value('TEMP',,'OS2ENVIRONMENT')
if TEMPDIR='' then TEMPDIR=value('TMP',,'OS2ENVIRONMENT')
if right(TEMPDIR,1)<>'\' then TEMPDIR=TEMPDIR'\'
TEMPDIR=TEMPDIR'bootAble.$$$'
CLEANUP_=CLEANUP
CLEANUP='NO'
rc=createDir(TEMPDIR)
CLEANUP=CLEANUP_


/****************************************************************************/
/**** Bootable CD creation **************************************************/
/****************************************************************************/

if isSet(CREATEBOOTCD) & isSet(CDRECORD) & isSet(CDRECORDCLEANDIR) then
    rc=cleanDir(WRITERDRIVE)

rc=prepareImage(1)
rc=copyImageFiles(1)
rc=createConfigSys()

/* Bootmessages? */
if isSet(BOOTMESSAGES) then
    rc=createFile(VIRTUALDRIVE'\OS2\BOOT\ALTF2ON.$$$', '')

if isSet(TWODISKBOOT) then do
    rc=writeImage(1)
    rc=prepareImage(2)
end

rc=copyImageFiles(2)

/* HPFS386 */
if isSet(HPFS386) then
    rc=createFile(VIRTUALDRIVE'\IBM386FS\HPFS386.INI', HPFS386INI)

if strip(STARTUPCMD)<>'' then
    rc=createFile(VIRTUALDRIVE'\STARTUP.CMD',STARTUPCMD)

if isSet(TWODISKBOOT) then
    rc=writeImage(2)
else
    rc=writeImage(1)

rc=copyCDFiles()

if isSet(WPS)        then rc=createINIFiles()
if isSet(NETWORKING) then rc=createNetworkFiles()

rc=createExtraCDFiles()

call progress 1, 'Done creating' ifSet(CREATEBOOTCD,'bootable CD', 'maintenance partition')
exit cleanUp()


/****************************************************************************/
/**** Functions *************************************************************/
/****************************************************************************/

/* Determine if an option is set ------------------------------------------ */
isSet: procedure
    return (left(translate(ARG(1)),1)='Y')


/* Inline version of the if clause ---------------------------------------- */
ifClause: procedure
    if (ARG(1)) then
        return ARG(2)
    else
        return ARG(3)


/* Combination of ifClause and isSet -------------------------------------- */
ifSet: procedure
    if (isSet(ARG(1))) then
        return ARG(2)
    else
        return ARG(3)


/* Returns the file argument if it exists, an empty string if not --------- */
/* If the second argument is not specified, the SEARCHPATH is used when the */
/*   file can not be found on the specified location                        */
/* Can be used in configuration files to add optional files                 */
fileExists: procedure expose SEARCHPATH
    FILESPEC=stream(ARG(1), 'C', 'QUERY EXISTS')
    if FILESPEC='' & ARG(2)='' & SEARCHPATH<>'' & pos('*', ARG(1))=0 & pos('?', ARG(1))=0 then do
        if SEARCHPATH<>'SEARCHPATH' then
            FILESPEC=sysSearchPath('SEARCHPATH', fileSpec('NAME', ARG(1)))
        else do
            call warningMessage 'Searching for' fileSpec('NAME', ARG(1))||': SEARCHPATH not set, using DPATH instead'
            FILESPEC=sysSearchPath('DPATH', fileSpec('NAME', ARG(1)))
        end
    end
    return FILESPEC


/* Returns the first file argument if it exists, the second if not -------- */
ifExists: procedure expose SEARCHPATH
    FILESPEC=fileExists(ARG(1))
    if FILESPEC<>'' then
        return FILESPEC
    else
        return ARG(2)


/* Returns the 1 if the directory exists, 0 if not  ----------------------- */
dirExists: procedure
    rc=sysFileTree(ARG(1), DIRS, 'D')
    return (rc=0 & DIRS.0=1)


/* Like sysSearchPath, but returns file argument if not found ------------- */
/* Function is to be used in configuration files when you want an error     */
/*   message for copying non-existing files                                 */
searchPath: procedure
    FULLNAME=sysSearchPath(ARG(1), ARG(2))
    if FULLNAME='' then
        FULLNAME=ARG(2)
    return FULLNAME


/* Return file name portion of a filespec --------------------------------- */
fileName: procedure
    if ARG(2) <> '' then do
        FS=filespec('NAME', ARG(1))
        FSDOT=pos('.', FS)
        if FSDOT>0 then do
            FSNAME=left(FS,min(FSDOT-1,8))
            FSEXT=right(FS,length(FS)-FSDOT)
        end
        else do
            FSNAME=left(FS,min(length(FS),8))
            FSEXT=''
        end
        return FSNAME||ifClause(FSDOT>0,'.')||FSEXT
    end
    else
        return filespec('NAME', ARG(1))


/* Return the value of a variable or an epmty string of not set ----------- */
varValue:
    if translate(arg(1))<>value(arg(1)) then
        return value(arg(1))
    else
        return '' 


/* Display progress when in given verbose mode ---------------------------- */
progress: procedure expose VERBOSE
    if VERBOSE>=ARG(1) then say ARG(2)
    return rc


/* Write an error message to stderr and exit program ---------------------- */
error: procedure expose CLEANUP TEMPDIR VERBOSE IGNOREOUTPUT
    call errorMessage ARG(1)
    rc=cleanUp()
    exit 1


/* Write an error message to stderr --------------------------------------- */
errorMessage: procedure
    call lineout 'STDERR', 'ERROR:' ARG(1)
    return 0


/* Write a warning message to stderr -------------------------------------- */
warningMessage: procedure
    call lineout 'STDERR', 'WARNING:' ARG(1)
    return 0


/* Cleanup ---------------------------------------------------------------- */
cleanUp: procedure expose CLEANUP TEMPDIR VERBOSE IGNOREOUTPUT
    rc=endLocal()
    if \ isSet(CLEANUP) then return 0
    rc=cleanDir(TEMPDIR,'YES')
    rc=sysRmDir(TEMPDIR)
    if rc<>0 then
        call errorMessage 'Failed to remove temp directory' TEMPDIR '('rc')'
    return rc


/* Clean a directory tree ------------------------------------------------- */
cleanDir: procedure expose CLEANUP TEMPDIR VERBOSE IGNOREOUTPUT
    call progress 1, 'Cleaning up directory' ARG(1)
    /* First delete all files */
    rc=sysFileTree(ARG(1)'\*', FILETREE, 'FOS',, '-*---')
    do I=1 to FILETREE.0 while rc=0
        rc=sysFileDelete(FILETREE.I)
    end
    if rc<>0 then
        ERR='Failed to remove files from directory' ARG(1) '('rc')'
    /* Then delete all directories */
    if rc=0 then do
        rc=sysFileTree(ARG(1)'\*', FILETREE, 'DOS',, '-*---')
        do I=FILETREE.0 to 1 by -1 while rc=0
            rc=sysRmDir(FILETREE.I)
        end
        if rc<>0 then
            ERR='Failed to remove directory structure' ARG(1) '('rc')'
    end
    if rc<>0 then
        if isSet(ARG(2)) then
            call errorMessage ERR
        else
            call error ERR
    return rc


/* Create a file with given content --------------------------------------- */
createFile: procedure expose CLEANUP TEMPDIR VERBOSE IGNOREOUTPUT
    FILE=ARG(1)
    TXT=ARG(2)
    call progress 2, 'Creating file' FILE
    rc=charout(FILE,TXT)
    if rc=0 then
        rc=charout(FILE)
    if rc<>0 then
        call error 'Failed to create' FILE
    return 0


/* Parse a list of files into FILELIST variable ---------------------------- */
parseFileList: procedure expose FILELIST.
    FILE=strip(ARG(1))
    I=0
    do while length(FILE)>0
        LFT=left(FILE,1)
        if LFT='"' then
            FILE=right(FILE,length(FILE)-1)
        else
            LFT=' '
        IDX=pos(LFT,FILE)
        I=I+1
        if IDX=0 then do
            FILELIST.I=FILE
            FILE=''
        end
        else do
            FILELIST.I=left(FILE,IDX-1)
            FILE=strip(right(FILE,length(FILE)-IDX))
        end
    end
    FILELIST.0=I
    return 0

/* Copy a list of files --------------------------------------------------- */
copyFiles: procedure expose SEARCHPATH CLEANUP TEMPDIR VERBOSE IGNOREOUTPUT LXLITE LXLITECOMMAND LXLITEOPTIONS
    DESTINATION=ARG(2)
    if right(DESTINATION,1)<>'\' then DESTINATION=DESTINATION'\'
    TRUNCATE=ARG(3)
    IGNORENOTFOUND=ARG(4)
    call progress 1, 'Copying files to' DESTINATION
    rc=parseFileList(ARG(1))
    do I=1 to FILELIST.0
        drop FILETREE.
        rc=sysFileTree(FILELIST.I, FILETREE, 'F')
        if rc=0 then do
            if FILETREE.0=0 & IGNORENOTFOUND='' then do
                /* If not found at specified location, search additional path */
                SOURCEFILE=fileExists(FILELIST.I)
                if SOURCEFILE<>'' then do
                    drop FILETREE.
                    rc=sysFileTree(SOURCEFILE, FILETREE, 'F')
                end
            end  
            if rc=0 then do 
                if FILETREE.0=0 & IGNORENOTFOUND='' then
                    call error 'File not found' FILELIST.I
                do II=1 to FILETREE.0
                    rc=copyFile(FILETREE.II, DESTINATION, TRUNCATE)
                end II
            end
        end
        if rc<>0 then
            call error 'Not enough memory copying' FILELIST.I
    end I
    return 0


/* Copy a file without extended attributes -------------------------------- */
copyFile: procedure expose CLEANUP TEMPDIR VERBOSE IGNOREOUTPUT LXLITE LXLITECOMMAND LXLITEOPTIONS
    parse value ARG(1) with dummy dummy dummy ATTRIBUTES SOURCEFILESPEC
    ATTRIBUTES=space(translate(ATTRIBUTES, ' ', '-'),0)
    SOURCEFILESPEC=strip(SOURCEFILESPEC)
    SOURCEDIR=fileSpec('PATH', SOURCEFILESPEC)
    SOURCEFILE=fileName(SOURCEFILESPEC)
    DESTDIR=ARG(2)
    DESTFILE=fileName(SOURCEFILE, ARG(3))
    if fileExists(DESTDIR||DESTFILE, 'T')<>'' then do
        call progress 2, 'Ignored' left(ATTRIBUTES,4) SOURCEFILESPEC '(file already copied)'
        return 0
    end
    call progress 2, 'Copying' left(ATTRIBUTES,4) SOURCEFILESPEC '=>' DESTDIR||DESTFILE
    'xcopy /h/t/r' '"'SOURCEFILESPEC'"' TEMPDIR IGNOREOUTPUT
    select
        when rc=1 then ERR='File not found'
        when rc=2 then ERR='File errors'
        when rc=3 then ERR='Interrupted'
        when rc=4 then ERR='Not enough room for file'
        otherwise ERR=''
    end
    if rc=0 then do
        'attrib -a -h -r -s' '"'TEMPDIR'\'SOURCEFILE'"' IGNOREOUTPUT
        if rc<>0 then ERR='Resetting attributes failed ('rc')'
    end
    if rc=0 then do
        'eautil /s /r' '"'TEMPDIR'\'SOURCEFILE'"' TEMPDIR'\eautil.$$$' IGNOREOUTPUT
        if rc<>0 then ERR='Removing extended attributes failed'
    end
    if rc=0 & isSet(LXLITE) then do
        LXLITECOMMAND LXLITEOPTIONS TEMPDIR'\'SOURCEFILE IGNOREOUTPUT
        if rc<>0 then ERR='Lxlite failed ('rc')'
    end
    if rc=0 then do
        'copy' '"'TEMPDIR'\'SOURCEFILE'"' '"'DESTDIR||DESTFILE'"' IGNOREOUTPUT
        if rc<>0 then ERR='Copy to' DESTDIR||DESTFILE 'failed ('rc')'
    end
    if rc=0 then do
        'del' '"'TEMPDIR'\'SOURCEFILE'"' IGNOREOUTPUT
        if rc<>0 then ERR='Deletion of' TEMPDIR'\'SOURCEFILE 'failed ('rc')'
    end
    if rc=0 & ATTRIBUTES<>'' then do
        do I=1 to length(ATTRIBUTES)
            SETATTRIBUTES=' +'||substr(ATTRIBUTES,I,1)
        end
        'attrib' SETATTRIBUTES '"'DESTDIR||DESTFILE'"' IGNOREOUTPUT
        if rc<>0 then ERR='Setting' ATTRIBUTES 'attributes failed ('rc')'
    end
    if rc<>0 then
        call error 'Error copying' SOURCEFILESPEC||':' ERR
    return rc


/* Create a directory ----------------------------------------------------- */
createDir: procedure expose CLEANUP TEMPDIR VERBOSE IGNOREOUTPUT
    DIRNAME=ARG(1)
    /* Don't create it if it already exists */
    if dirExists(DIRNAME) then do
        call progress 2, 'Ignored directory' DIRNAME '(already created)'
        return 0
    end

    /* Make sure all higher directory levels are created */
    IDX=lastpos('\',DIRNAME)
    if IDX>3 then rc=createDir(left(DIRNAME,IDX-1))

    /* Create directory */
    call progress 2, 'Creating directory' DIRNAME
    rc=sysMkDir(DIRNAME)
    select
        when rc=2   then ERR='File not found'
        when rc=3   then ERR='Path not found'
        when rc=5   then ERR='Access denied'
        when rc=26  then ERR='Not a DOS disk'
        when rc=87  then ERR='Invalid parameter'
        when rc=108 then ERR='Drive locked'
        when rc=206 then ERR='Filename exceeds range'
        otherwise ERR=''
    end
    if rc<>0 then
        call error 'Error creating directory' DIRNAME||':' ERR
    return rc


/* Write a line to the config.sys file ------------------------------------ */
configLine: procedure expose VIRTUALDRIVE
    rc=lineout(VIRTUALDRIVE'\config.sys',ARG(1))
    return rc


/* Read and parse the configuration file(s) ------------------------------- */
readConfig:
    rc=parseFileList(ARG(1))
    PREVSEARCHPATH=SEARCHPATH

    /* Process all specified configuration files */
    do I=1 to FILELIST.0
        rc=readConfigFile(FILELIST.I)
    end
    /* Process the defaults */
    rc=readConfigFile('bootAble.cfg', 'YES')

    /* Make sure the RESERVEDRIVELETTER has no trailing ':' */
    RESERVEDRIVELETTER=left(RESERVEDRIVELETTER,1)

    /* Force serial port settings */
    if isSet(SIO2KSERIALPORT) then do
        SIOSERIALPORT='NO'
        SERIALPORT='NO'
    end
    else do
        SIO2KPATCH='NO'
        if isSet(SIOSERIALPORT) then
            SERIALPORT='NO'
    end

    /* Force SCSI settings */
    if SCSI=0 then do
        ASPI='NO'
        REMOVABLESCSI='NO'
    end

    /* Force USB settings */
    if (USBOHCI>0 | USBUHCI>0 | USBEHCI>0) then
        USB='YES'
    else do
        USB='NO'
        USBKEYB='NO'
        USBCOM='NO'
        USBMOUSE='NO'
        USBPRINTER='NO'
        USBCDROM='NO'
        USBSTORAGE='NO'
    end

    /* Force WPS settings */
    WPSDRIVE=WRITERDRIVE
    if isSet(WPS) then do
        if isSet(CREATEBOOTCD) then do
            RAMDISK='YES'
            WPSDRIVE=RAMDISKDRIVE
        end
        MOUSE='YES'
    end
    else do
        WPSBOOT='NO'
        WPSAPPS='NO'
    end

    /* Networking? */
    if isSet(NETBIOSTCPIP) then
        TCPIP='YES'
    if isSet(TCPIP) | isSet(NETBIOS) then
        NETWORKING='YES'
    else
        NETWORKING='NO'
    if \ isSet(TCPIP) then do
        TCPIPDHCP='NO'
        TCPIP431='NO'
    end

    /* Force extra directory settings */
    do I=1 to EXTRADIRECTORIES
        IDX=pos(":", EXTRADIRECTORY.I)
	if IDX<>0 then EXTRADIRECTORY.I=right(EXTRADIRECTORY.I, length(EXTRADIRECTORY.I)-IDX)
        if left(EXTRADIRECTORY.I,1)<>'\' then EXTRADIRECTORY.I='\'EXTRADIRECTORY.I
        EXTRADIRECTORYADDTOPATHS.I=translate(EXTRADIRECTORYADDTOPATHS.I)
        EXTRADIRECTORYADDTOPATHS.I=translate(EXTRADIRECTORYADDTOPATHS.I, '  ', ',;')
        EXTRADIRECTORYADDTOPATHS.I=strip(EXTRADIRECTORYADDTOPATHS.I)
    end

    /* Force extra create file settings */
    do I=1 to EXTRACREATEFILES
        IDX=pos(":", EXTRACREATEFILEDIRECTORY.I)
	if IDX<>0 then EXTRACREATEFILEDIRECTORY.I=right(EXTRACREATEFILEDIRECTORY.I, length(EXTRACREATEFILEDIRECTORY.I)-IDX)
        if left(EXTRACREATEFILEDIRECTORY.I,1)<>'\' then EXTRACREATEFILEDIRECTORY.I='\'EXTRACREATEFILEDIRECTORY.I
    end

    /* Force settings when creating a maintenance partition */
    if \ isSet(CREATEBOOTCD) then do
        VIRTUALDRIVE=WRITERDRIVE
        CDROMDRIVE=WRITERDRIVE
        MAKEROOM='NO'
        TWODISKBOOT='NO'
    end

    /* Only one of HPFS386 and HPFS can be set */
    if isSet(HPFS386) then HPFS='NO'

    /* Force AUTOCHECK options */
    if \ isSet(HPFS)    then HPFSAUTOCHECK='NO'
    if \ isSet(HPFS386) then HPFS386AUTOCHECK='NO'
    if \ isSet(JFS)     then JFSAUTOCHECK='NO'
    if isSet(HPFSAUTOCHECK) | isSet(JFSAUTOCHECK) | isSet(HPFS386AUTOCHECK) then
        AUTOCHECK='YES'
    else
        AUTOCHECK='NO'

    return 0


/* Read and parse an individual configuration file ------------------------ */
readConfigFile:
    CONFIGFILE=ARG(1)
    SETDEFAULTS=ARG(2)
    CONFIGFILE=fileExists(CONFIGFILE)
    if CONFIGFILE='' then 
        call error 'Configuration file' ARG(1) 'not found'
    STATEMENT=''
    LINENR=0
    do while lines(CONFIGFILE)>0
        LINE=strip(linein(CONFIGFILE))
        LINENR=LINENR+1
        CONT=0
        if right(LINE,1)=',' then do
            CONT=1
            LINE=left(LINE,length(LINE)-1)
        end
        if length(LINE)>0 & left(LINE,2)<>'/*' & left(LINE,1)<>';' then
            STATEMENT=strip(STATEMENT LINE)
        if CONT=0 then do
            DOPARSE='YES'
            if isSet(SETDEFAULTS) then do
                EQUALSIGN=pos('=', STATEMENT)
                if (EQUALSIGN>0) then do
                    VARNAME=left(STATEMENT,EQUALSIGN-1)
                    if translate(VARNAME)<>value(VARNAME) then
                        DOPARSE='NO'
                end
            end
            if isSet(DOPARSE) then do
                interpret STATEMENT
                /* Searchpath set? */
                if SEARCHPATH<>PREVSEARCHPATH then do
                    PREVSEARCHPATH=SEARCHPATH
                    rc=value('SEARCHPATH',SEARCHPATH,'OS2ENVIRONMENT')
                end
            end
            STATEMENT=''
        end
    end
    rc=stream(CONFIGFILE, 'C', 'CLOSE')
    if rc<>'READY:' then
        call error 'Failed to close' CONFIGFILE 'rc='rc

    return 0


/* List of MAKEROOM files for the os2 directory --------------------------- */
makeRoomFiles:
    FILES_=VITALFILES_ SHELLFILE PROTSHELLFILE
    if isSet(WPS)               then FILES_=FILES_ WPSRUNWORKPLACEFILE
    if isSet(CDFS)              then FILES_=FILES_ CDFSFILES
    if isSet(FAT32)             then FILES_=FILES_ FAT32FILES FAT32CACHEFILE
    if isSet(NTFS)              then FILES_=FILES_ NTFSFILES
    if isSet(UDF)               then FILES_=FILES_ UDFFILES
    if isSet(RAMDISK)           then FILES_=FILES_ RAMDISKFILE
    if isSet(APM)               then FILES_=FILES_ APMFILE APMDAEMONFILE
    if isSet(SERIALPORT)        then FILES_=FILES_ SERIALPORTFILE
    if isSet(SIOSERIALPORT)     then FILES_=FILES_ SIOSERIALPORTFILE
    if isSet(SIO2KSERIALPORT)   then FILES_=FILES_ SIO2KUARTFILE SIO2KSERIALPORTFILE
    if isSet(SIO2KPATCH)        then FILES_=FILES_ SIO2KPATCHFILE
    if isSet(MOUSE)             then FILES_=FILES_ MOUSEPOINTDDFILE MOUSEFILE MOUSEDEVFILE
    if isSet(USBKEYB)           then FILES_=FILES_ USBKEYBFILE
    if isSet(USBCOM)            then FILES_=FILES_ USBCOMFILE
    if isSet(USBMOUSE)          then FILES_=FILES_ USBMOUSEFILE
    if isSet(USBPRINTER)        then FILES_=FILES_ USBPRINTERFILE
    if isSet(LVM)               then FILES_=FILES_ LVMFILES
    if isSet(HPFS)    & \ isSet(HPFSAUTOCHECK)    then FILES_=FILES_ HPFSFILES
    if isSet(HPFS386) & \ isSet(HPFS386AUTOCHECK) then FILES_=FILES_ HPFS386OS2FILES
    if isSet(JFS)     & \ isSet(JFSAUTOCHECK)     then FILES_=FILES_ JFSFILES
    if                  \ isSet(AUTOCHECK)        then FILES_=FILES_ VITALFILES__
    return FILES_


/* List of AUTOCHECK files for the os2 directory -------------------------- */
autoCheckFiles:
    FILES__=''
    if isSet(HPFS)    & isSet(HPFSAUTOCHECK)    then FILES__=FILES__ HPFSFILES
    if isSet(HPFS386) & isSet(HPFS386AUTOCHECK) then FILES__=FILES__ HPFS386OS2FILES
    if isSet(JFS)     & isSet(JFSAUTOCHECK)     then FILES__=FILES__ JFSFILES
    if                  isSet(AUTOCHECK)        then FILES__=FILES__ VITALFILES__
    return FILES__


/* List of MAKEROOM files for the os2\dll directory ----------------------- */
makeRoomDLLFiles:
    DLLFILES_=VITALDLLFILES_
    if isSet(CDFS)  then DLLFILES_=DLLFILES_ CDFSDLLFILES
    if isSet(FAT32) then DLLFILES_=DLLFILES_ FAT32DLLFILES
    if isSet(NTFS)  then DLLFILES_=DLLFILES_ NTFSDLLFILES
    if isSet(UDF)   then DLLFILES_=DLLFILES_ UDFDLLFILES
    if isSet(HPFS)    & \ isSet(HPFSAUTOCHECK)    then DLLFILES_=DLLFILES_ HPFSDLLFILES
    if isSet(HPFS386) & \ isSet(HPFS386AUTOCHECK) then DLLFILES_=DLLFILES_ HPFS386OS2DLLFILES
    if isSet(JFS)     & \ isset(JFSAUTOCHECK)     then DLLFILES_=DLLFILES_ JFSDLLFILES
    if isSet(LVM)     & \ isSet(AUTOCHECK)        then DLLFILES_=DLLFILES_ LVMDLLFILES
    if isSet(UNICODE) & \ isSet(AUTOCHECK)        then DLLFILES_=DLLFILES_ UNICODEDLLFILES
    if                  \ isSet(AUTOCHECK)        then DLLFILES_=DLLFILES_ VITALDLLFILES__
    return DLLFILES_
    

/* List of AUTOCHECK files for the os2\dll directory ---------------------- */
autoCheckDLLFiles:
    DLLFILES__=''
    if isSet(HPFS)    & isSet(HPFSAUTOCHECK)    then DLLFILES__=DLLFILES__ HPFSDLLFILES
    if isSet(HPFS386) & isSet(HPFS386AUTOCHECK) then DLLFILES__=DLLFILES__ HPFS386OS2DLLFILES
    if isSet(JFS)     & isset(JFSAUTOCHECK)     then DLLFILES__=DLLFILES__ JFSDLLFILES
    if isSet(LVM)     & isSet(AUTOCHECK)        then DLLFILES__=DLLFILES__ LVMDLLFILES
    if isSet(UNICODE) & isSet(AUTOCHECK)        then DLLFILES__=DLLFILES__ UNICODEDLLFILES
    if                  isSet(AUTOCHECK)        then DLLFILES__=DLLFILES__ VITALDLLFILES__
    return DLLFILES__
    

/* Prepare a boot image --------------------------------------------------- */
prepareImage:
    IMAGENR=ARG(1)

    /* Format the (virtual) disk */
    call progress 1, 'Formatting boot image' IMAGENR
    if isSet(CREATEBOOTCD) | isSet(FORMATMAINTENANCE) then do
        LABEL=ifSet(CREATEBOOTCD,'CDBOOT_'IMAGENR,'MAINTENANCE')
        'label' VIRTUALDRIVE||'FAILED' IGNOREOUTPUT
        if rc<>0 then
            call error 'Failed to reset the label of' VIRTUALDRIVE
        rc=lineOut(TEMPDIR'\response', 'FAILED'||CRLF||'Y')
        if rc=0 then
            rc=lineout(TEMPDIR'\response')
        if rc<> 0 then
            call error 'Failed to create format response file'
        'format' VIRTUALDRIVE '/fs:'ifSet(CREATEBOOTCD,'fat','hpfs') '/once /v:'LABEL '<' TEMPDIR'\response' IGNOREOUTPUT
        /* Somehow format does not always return non-zero rc in case of error... */
        if rc<>0 | word(sysDriveInfo(VIRTUALDRIVE),4)<>LABEL then
            call error 'Error formatting boot image' IMAGENR '('VIRTUALDRIVE')'
    end

    /* Install system files */
    call progress 1, 'Transfering system files to boot image' IMAGENR
    SYSINSTX VIRTUALDRIVE IGNOREOUTPUT
    if rc<>0 then
        call error 'Error transfering system files to boot image' IMAGENR

    return 0


/* Write a boot image to CD ----------------------------------------------- */
writeImage:
    if \ isSet(CREATEBOOTCD) then return 0

    IMAGENR=ARG(1)

    if IMAGENR=1 & isSet(CDBOOTMENU) then do
        CDBOOTMENUFILE '/DRIVE' left(VIRTUALDRIVE,1) '/TIMEOUT' CDBOOTMENUTIMEOUT CDBOOTMENUOTHEROPTIONS IGNOREOUTPUT
        if rc<>0 then
            call error 'Failed adding boot menu'
    end

    if isSet(PAUSEBEFOREBOOTIMAGE) then do
        say 'Pausing before writing boot image' IMAGENR
        say 'Continue with script? ([Y]es/[N]o/[Q]uit without cleaning up)'
        rc=1
        do until rc=0
            KEY=translate(sysGetKey('NOECHO'))
            if pos(KEY,'YNQ')>0 then
                rc=0
            else
                call beep 524,100
        end
        say KEY
        if KEY<>'Y' then do
            if KEY='Q' then
                CLEANUP='NO'
            call error 'Script aborted'
        end
    end

    call progress 1, 'Writing boot image' IMAGENR
    'savedskf /a/d' VIRTUALDRIVE WRITERDRIVE"\"BOOTIMAGENAME.IMAGENR IGNOREOUTPUT
    if rc<>0 then
        call error 'Error writing boot image' IMAGENR

    /* The first image will be marked bootable */
    if IMAGENR=1 & \ isSet(CDRECORD) then do
        'cdboot' WRITERDRIVE||BOOTIMAGENAME.IMAGENR IGNOREOUTPUT
        if rc<>0 then
            call error 'Error marking boot image'
    end

    return 0


/* Copy files to boot image ----------------------------------------------- */
copyImageFiles:
    IMAGENR=ARG(1)

    call progress 1, 'Copying files to boot image' IMAGENR

    /* Files to copy to root directory of boot image */
    if IMAGENR=1 | isSet(TWODISKBOOT) then
        rc=copyFiles(VITALROOTFILES.IMAGENR, VIRTUALDRIVE'\', 'T')

    /* Files to copy to os2 directory of boot image */
    if IMAGENR=1 | isSet(TWODISKBOOT) then do
        rc=createDir(VIRTUALDRIVE'\OS2')
    end
    if IMAGENR=2 then do
        FILES=VITALFILES
        if   isSet(CDROM)        then FILES=FILES CDROMDMDFILE
        if   isSet(CDFS)         then FILES=FILES CDFSIFSFILE
        if   isSet(FAT32)        then FILES=FILES FAT32IFSFILE
        if   isSet(NTFS)         then FILES=FILES NTFSIFSFILE
        if   isSet(HPFS)         then FILES=FILES HPFSIFSFILE
        if   isSet(JFS)          then FILES=FILES JFSIFSFILE
        if   isSet(UDF)          then FILES=FILES UDFIFSFILE
        if   isSet(RAMDISK)      then FILES=FILES RAMDISKIFSFILE
        if   isSet(UNICODE)      then FILES=FILES UNICODEFILE
        if   isSet(AUTOCHECK)    then FILES=FILES autoCheckFiles()
        if \ isSet(MAKEROOM)     then FILES=FILES makeRoomFiles()
        rc=copyFiles(FILES, VIRTUALDRIVE'\os2', 'T')
    end
    
    /* Files to copy to os2\boot directory of boot image */
    if IMAGENR=1 then do
        rc=createDir(VIRTUALDRIVE'\OS2\BOOT')
        BOOTFILES=VITALBOOTFILES DASDFILE
        if isSet(CREATEBOOTCD)  then BOOTFILES=BOOTFILES BOOTFLTFILE
        if isSet(WPS)           then BOOTFILES=BOOTFILES WPSBOOTFILES
        if isSet(FLOPPY)        then BOOTFILES=BOOTFILES FLOPPYADDFILE
        if isSet(PARALLELPORT)  then BOOTFILES=BOOTFILES PARALLELPORTFILE
        if isSet(PPZIP)         then BOOTFILES=BOOTFILES PPZIPADDFILE
        if isSet(NON512DASD)    then BOOTFILES=BOOTFILES NON512DASDFILE
        if isSet(DASDFALLBACK)  then BOOTFILES=BOOTFILES DASDFALLBACKFILE
        if isSet(IDE)           then BOOTFILES=BOOTFILES IDEADDFILE
        if isSet(ATAPI)         then BOOTFILES=BOOTFILES ATAPIFLTFILE ifClause(\isSet(DANIATAPI)&\isSet(LATESTATAPI),ATAPICDFLTFILE)
        if isSet(LVM)           then BOOTFILES=BOOTFILES LVMDMDFILE
        if isSet(ASPI)          then BOOTFILES=BOOTFILES ASPIDMDFILE
        if isSet(REMOVABLESCSI) then BOOTFILES=BOOTFILES REMOVABLESCSIFILE
        do I=1 to SCSI
            BOOTFILES=BOOTFILES SCSIADDFILE.I
        end
        do I=1 to EXTRABASEDEV
            BOOTFILES=BOOTFILES EXTRABASEDEVFILE.I
        end
        if isSet(USB) then do
            BOOTFILES=BOOTFILES USBDFILE USBHIDFILE
            if USBOHCI>0 then BOOTFILES=BOOTFILES USBOHCIFILE
            if USBUHCI>0 then BOOTFILES=BOOTFILES USBUHCIFILE
            if USBEHCI>0 then BOOTFILES=BOOTFILES USBEHCIFILE
            if isSet(USBCDROM)   then BOOTFILES=BOOTFILES USBCDROMFILE
            if isSet(USBSTORAGE) then BOOTFILES=BOOTFILES USBSTORAGEFILE
        end
        rc=copyFiles(BOOTFILES, VIRTUALDRIVE'\os2\boot', 'T')
    end
    if IMAGENR=2 & isSet(APM) & APMGSVDAEMONFILE<>'' then do
        if isSet(TWODISKBOOT) then
            rc=createDir(VIRTUALDRIVE'\OS2\BOOT')
        rc=copyFiles(APMGSVDAEMONFILE, VIRTUALDRIVE'\OS2\BOOT', 'T')
    end

    /* Files to copy to os2\dll directory of boot image */
    if IMAGENR=2 then do
        rc=createDir(VIRTUALDRIVE'\OS2\DLL')
        DLLFILES=VITALDLLFILES
        if   isSet(AUTOCHECK) then DLLFILES=DLLFILES autoCheckDLLFiles()
        if \ isSet(MAKEROOM)  then DLLFILES=DLLFILES makeRoomDLLFiles()
        rc=copyFiles(DLLFILES, VIRTUALDRIVE'\os2\dll', 'T')
    end
    
    /* Unicode support files */
    if IMAGENR=2 & isSet(UNICODE) then do
        /* Language directory tree */
        rc=createDir(VIRTUALDRIVE'\LANGUAGE')
        rc=createDir(VIRTUALDRIVE'\LANGUAGE\CODEPAGE')
        rc=createDir(VIRTUALDRIVE'\LANGUAGE\FONT')
        rc=createDir(VIRTUALDRIVE'\LANGUAGE\KEYBOARD')
        rc=createDir(VIRTUALDRIVE'\LANGUAGE\LOCALE')
        rc=copyFiles(UNICODECODEPAGEFILE, VIRTUALDRIVE'\language\codepage', 'T')
        rc=copyFiles(UNICODEKEYBOARDFILE, VIRTUALDRIVE'\language\keyboard', 'T')
        rc=copyFiles(UNICODEUSERFILE,     VIRTUALDRIVE'\language\locale',   'T')
    end

    /* HPFS386 support files */
    if IMAGENR=2 & isSet(HPFS386) then do
        rc=createDir(VIRTUALDRIVE'\IBM386FS')
        rc=copyFiles(HPFS386IFSFILE HPFS386FILES, VIRTUALDRIVE'\ibm386fs', 'T')
    end

    return rc


/* Copy files to CD ------------------------------------------------------- */
copyCDFiles:
    call progress 1, 'Writing files to' ifSet(CREATEBOOTCD,'CD','Maintenance Partition')

    /* Files to copy to os2 directory of CD */
    rc=createDir(WRITERDRIVE'\OS2')
    FILES=EXTRAFILES
    if isSet(OS2APPS)  then FILES=FILES OS2APPFILES
    if isSet(MAKEROOM) then FILES=FILES makeRoomFiles()
    if isSet(REXX)     then FILES=FILES REXXFILES ifSet(WPSBOOT,'',REXXCOMMANDLINEFILE)
    if isSet(WPS)      then FILES=FILES WPSFILES VIDEODRIVERFILES.VIDEODRIVER
    do I=1 to EXTRADEVICE
        FILES=FILES EXTRADEVICEFILE.I
    end
    rc=copyFiles(FILES, WRITERDRIVE'\os2')

    /* Files to copy to os2\dll directory of CD */
    rc=createDir(WRITERDRIVE'\OS2\DLL')
    DLLFILES=EXTRADLLFILES
    if isSet(OS2APPS)                     then DLLFILES=DLLFILES OS2APPDLLFILES
    if isSet(MAKEROOM)                    then DLLFILES=DLLFILES makeRoomDLLFiles()
    if isSet(REXX)                        then DLLFILES=DLLFILES REXXDLLFILES
    if isSet(WPS)                         then DLLFILES=DLLFILES WPSDLLFILES VIDEODRIVERDLLFILES.VIDEODRIVER
    if isSet(TCPIP431) & isSet(TCPIPDHCP) then DLLFILES=DLLFILES TCPIP431WPSDLLFILES
    rc=copyFiles(DLLFILES, WRITERDRIVE'\os2\dll')

    /* Files to copy to os2\help directory of CD */
    if isSet(WPS) then do
        rc=createDir(WRITERDRIVE'\OS2\HELP')
        HELPFILES=WPSHELPFILES VIDEODRIVERHELP.VIDEODRIVER EXTRAHELPFILES
        if isSet(WPSAPPS)      then HELPFILES=HELPFILES WPSAPPSHELPFILES
        if isSet(REXX)         then HELPFILES=HELPFILES REXXHELPFILES
        if isSet(TCPIP)        then HELPFILES=HELPFILES TCPIPHELPFILES
        if isSet(NETBIOS)      then HELPFILES=HELPFILES NETBIOSHELPFILES
        if isSet(NETBIOSTCPIP) then HELPFILES=HELPFILES NETBIOSTCPIPHELPFILES
        rc=copyFiles(HELPFILES, WRITERDRIVE'\os2\help')
    end

    /* Files to copy to os2\apps directory of CD */
    if isSet(WPSAPPS) then do
        rc=createDir(WRITERDRIVE'\OS2\APPS')
        rc=copyFiles(WPSAPPSFILES, WRITERDRIVE'\OS2\APPS')
    end

    /* Files to copy to os2\apps\dll directory of CD */
    if isSet(WPSAPPS) then do
        rc=createDir(WRITERDRIVE'\OS2\APPS\DLL')
        rc=copyFiles(WPSAPPSDLLFILES, WRITERDRIVE'\OS2\APPS\DLL')
    end

    /* Files to copy to network directory of CD */
    if isSet(NETWORKING) then do
        rc=createDir(WRITERDRIVE'\NETWORK')
        FILES=NETWORKCARDFILE NETWORKINGFILES
        if isSet(TCPIP)        then FILES=FILES TCPIPFILES
        if isSet(NETBIOS)      then FILES=FILES NETBIOSFILES
        if isSet(NETBIOSTCPIP) then FILES=FILES NETBIOSTCPIPFILES
        rc=copyFiles(FILES,WRITERDRIVE'\NETWORK')
    end

    /* Files to copy to network\bin directory of CD */
    if isSet(NETWORKING) then do
        rc=createDir(WRITERDRIVE'\NETWORK\BIN')
        BINFILES=NETWORKINGBINFILES
        if isSet(TCPIP)        then BINFILES=BINFILES TCPIPBINFILES
        if isSet(NETBIOS)      then BINFILES=BINFILES NETBIOSBINFILES
        if isSet(NETBIOSTCPIP) then BINFILES=BINFILES NETBIOSTCPIPBINFILES
        rc=copyFiles(BINFILES,WRITERDRIVE'\NETWORK\BIN')
    end

    /* Files to copy to network\dll directory of CD */
    if isSet(NETWORKING) then do
        rc=createDir(WRITERDRIVE'\NETWORK\DLL')
        DLLFILES=NETWORKINGDLLFILES
        if isSet(TCPIP)                       then DLLFILES=DLLFILES TCPIPDLLFILES
        if isSet(TCPIP431) & isSet(TCPIPDHCP) then DLLFILES=DLLFILES TCPIP431DLLFILES
        if isSet(NETBIOS)                     then DLLFILES=DLLFILES NETBIOSDLLFILES
        if isSet(NETBIOSTCPIP)                then DLLFILES=DLLFILES NETBIOSTCPIPDLLFILES
        rc=copyFiles(DLLFILES,WRITERDRIVE'\NETWORK\DLL')
    end

    /* Extra directories */
    do I=1 to EXTRADIRECTORIES
        rc=createDir(WRITERDRIVE||EXTRADIRECTORY.I)
        rc=copyFiles(EXTRADIRECTORYFILES.I,WRITERDRIVE||EXTRADIRECTORY.I,,ifSet(EXTRADIRECTORYRECURSE.I,'I',''))
        if isSet(EXTRADIRECTORYRECURSE.I) then do
            rc=parseFileList(EXTRADIRECTORYFILES.I)
            drop EDFILELIST.
            EDFILELIST.0=FILELIST.0
            do II=1 to FILELIST.0
                EDFILELIST.II=FILELIST.II
            end
            do II=1 to EDFILELIST.0
                BASEDIR=filespec('Drive', EDFILELIST.II)||filespec('Path', EDFILELIST.II)
                if substr(BASEDIR,2,2)<>':\' then 
                    call warningMessage 'Ignored invalid recursive extra directory specification' EDFILELIST.II 
                else do
                    FILEMASK=filespec('Name', EDFILELIST.II)
                    rc=sysFileTree(BASEDIR'*', DIRTREE, 'DOS')
                    if rc<>0 then call error 'Failed to retrieve directory tree for recursive extra directory' EDFILELIST.II
                    do III=1 to DIRTREE.0
                        NEWDIR=EXTRADIRECTORY.I||substr(DIRTREE.III,length(BASEDIR))
                        rc=createDir(WRITERDRIVE||NEWDIR)
                        rc=copyFiles('"'||DIRTREE.III||'\'||FILEMASK||'"',WRITERDRIVE||NEWDIR,,'I')
                    end
                end
            end
        end     
    end

    return rc


/* Create config.sys file ------------------------------------------------- */
createConfigSys:
    call progress 1, 'Creating config.sys file on boot image'
    
    /* Drive letter for files that can reside on boot image or on CD */
    if isSet(MAKEROOM) | \ isSet(CREATEBOOTCD) then
        DRIVELETTER=CDROMDRIVE
    else
        DRIVELETTER='A:'

    /* Truncate file names only if they are written to the boot image */
    if isSet(MAKEROOM) then
        TRUNCATE=''
    else
        TRUNCATE='T'

    /* Reservedriveletter */
    rc=configLine('RESERVEDRIVELETTER='RESERVEDRIVELETTER)

    /* Paths */
    LIBPATHLINE='LIBPATH=.;'||,
        ifSet(CREATEBOOTCD,'A:\os2\dll;')||,
        CDROMDRIVE'\os2\dll;'||,
        ifSet(WPSAPPS,CDROMDRIVE'\os2\apps\dll;')||,
        ifSet(NETWORKING,CDROMDRIVE'\network\dll;')||,
        ifSet(HPFS386,ifSet(CREATEBOOTCD,'A:',CDROMDRIVE)||'\ibm386fs;')
    PATHLINE='SET PATH='||,
        ifSet(CREATEBOOTCD,'A:\os2;')||,
        CDROMDRIVE'\os2;'||,
        ifSet(WPSAPPS,CDROMDRIVE'\os2\apps;')||,
        ifSet(NETWORKING,CDROMDRIVE'\network\bin;')||,
        ifSet(HPFS386,ifSet(CREATEBOOTCD,'A:',CDROMDRIVE)||'\ibm386fs;')
    DPATHLINE='SET DPATH='||,
        ifSet(CREATEBOOTCD,'a:\os2;')||,
        CDROMDRIVE'\os2;'||,
        ifSet(WPSAPPS,CDROMDRIVE'\os2\apps;')||,
        ifSet(NETWORKING,CDROMDRIVE'\network;')||,
        ifSet(HPFS386,ifSet(CREATEBOOTCD,'A:',CDROMDRIVE)||'\ibm386fs;')
    HELPPATHLINE='SET HELP='CDROMDRIVE'\os2\help;'
    BOOKSHELFLINE='SET BOOKSHELF='CDROMDRIVE'\os2\help;'

    do I=1 to EXTRADIRECTORIES
        ADDTOPATHS=EXTRADIRECTORYADDTOPATHS.I
        do while words(ADDTOPATHS)>0
            ADDTOPATH=word(ADDTOPATHS,1)
            ADDTOPATHS=strip(delword(ADDTOPATHS,1,1))
            select
                when ADDTOPATH='LIBPATH'    then LIBPATHLINE=LIBPATHLINE||CDROMDRIVE||EXTRADIRECTORY.I||';'
                when ADDTOPATH='PATH'       then PATHLINE=PATHLINE||CDROMDRIVE||EXTRADIRECTORY.I||';'
                when ADDTOPATH='DPATH'      then DPATHLINE=DPATHLINE||CDROMDRIVE||EXTRADIRECTORY.I||';'
                when ADDTOPATH='HELP'       then HELPPATHLINE=HELPPATHLINE||CDROMDRIVE||EXTRADIRECTORY.I||';'
                when ADDTOPATH='BOOKSHELF'  then BOOKSHELFLINE=BOOKSHELFLINE||CDROMDRIVE||EXTRADIRECTORY.I||';'
                otherwise ;
            end
        end
    end
    rc=configLine(LIBPATHLINE)
    rc=configLine(PATHLINE)
    rc=configLine(DPATHLINE)
    if isSet(WPS) then do
        rc=configLine(HELPPATHLINE)
        rc=configLine(BOOKSHELFLINE)
    end
    

    /* File systems */
    if isSet(CDFS)    then rc=configLine('IFS=\os2\'fileName(CDFSIFSFILE,'T')   CDFSOPTIONS)
    if isSet(FAT32)   then rc=configLine('IFS=\os2\'fileName(FAT32IFSFILE,'T')  FAT32OPTIONS)
    if isSet(FAT32)   then rc=configLine('CALL='DRIVELETTER'\os2\'fileName(FAT32CACHEFILE,'T') FAT32CACHEOPTIONS)
    if isSet(NTFS)    then rc=configLine('IFS=\os2\'fileName(NTFSIFSFILE,'T')    NTFSOPTIONS)
    if isSet(HPFS)    then rc=configLine('IFS=\os2\'fileName(HPFSIFSFILE,'T')    HPFSOPTIONS)
    if isSet(HPFS386) then rc=configLine('IFS=\ibm386fs\'fileName(HPFS386IFSFILE,'T') HPFS386OPTIONS)
    if isSet(JFS)     then rc=configLine('IFS=\os2\'fileName(JFSIFSFILE,'T')     JFSOPTIONS)
    if isSet(UDF)     then rc=configLine('IFS=\os2\'fileName(UDFIFSFILE,'T')     UDFOPTIONS)

    /* Ramdisk */    
    if isSet(RAMDISK) then do
        rc=configLine('IFS=\os2\'fileName(RAMDISKIFSFILE,'T') RAMDISKOPTIONS)
        rc=configLine('RUN='DRIVELETTER'\os2\'filename(RAMDISKFILE,TRUNCATE) RAMDISKDRIVE 'RAMDISK')
        rc=configLine('CALL='filename(SHELLFILE,TRUNCATE) '/c mkdir' RAMDISKDRIVE'\Temp > /dev/nul' )
        rc=configLine('SET TEMP='RAMDISKDRIVE'\Temp')
        rc=configLine('SET TMP='RAMDISKDRIVE'\Temp')
        rc=configLine('SUPPRESSPOPUPS='left(RAMDISKDRIVE,1))
    end

    /* Base device driver for keyboard */
    rc=configLine('BASEDEV=ibmkbd.sys')

    /* Base device for chkdsk large disk support */
    rc=configLine('BASEDEV=chkdsk.sys')

    /* Non-512 bytes/sector removable media support */
    if isSet(NON512DASD) then rc=configLine('BASEDEV='fileName(NON512DASDFILE) NON512DASDOPTIONS)

    /* Removable SCSI devices geometry filter support */
    if isSet(REMOVABLESCSI) then rc=configLine('BASEDEV='fileName(REMOVABLESCSIFILE,'T') REMOVABLESCSIOPTIONS)

    /* Base device driver for storage */
    rc=configLine('BASEDEV='fileName(DASDFILE,'T') DASDOPTIONS)

    /* Base device drivers for storage devices */
    if isSet(FLOPPY) then rc=configLine('BASEDEV='fileName(FLOPPYADDFILE,'T') FLOPPYOPTIONS)
    if isSet(PPZIP)  then rc=configLine('BASEDEV='fileName(PPZIPADDFILE,'T')  PPZIPOPTIONS)
    if isSet(IDE) & \ isSet(SCSIBEFOREIDE) then rc=configLine('BASEDEV='fileName(IDEADDFILE,'T') IDEOPTIONS)
    do I=1 to SCSI
        rc=configLine('BASEDEV='fileName(SCSIADDFILE.I,'T') SCSIOPTIONS.I)
    end
    if isSet(IDE) & isSet(SCSIBEFOREIDE) then rc=configLine('BASEDEV='fileName(IDEADDFILE,'T') IDEOPTIONS)
    if isSet(ATAPI) then do
        rc=configLine('BASEDEV='fileName(ATAPIFLTFILE,'T') ATAPIOPTIONS)
        if \ isSet(DANIATAPI) & \ isSet(LATESTATAPI) then rc=configLine('BASEDEV='fileName(ATAPICDFLTFILE,'T') ATAPICDOPTIONS)
    end
    if isSet(DASDFALLBACK) then rc=configLine('BASEDEV='fileName(DASDFALLBACKFILE,'T') DASDFALLBACKOPTIONS)
    if isSet(LVM)          then rc=configLine('BASEDEV='fileName(LVMDMDFILE,'T')       LVMOPTIONS)

    /* Extra base device drivers */
    do I=1 to EXTRABASEDEV
        rc=configLine('BASEDEV='fileName(EXTRABASEDEVFILE.I,'T') EXTRABASEDEVOPTIONS.I)
    end
    
    /* Base device driver providing bootable CD support */
    if isSet(CREATEBOOTCD) then rc=configLine('BASEDEV='fileName(BOOTFLTFILE,'T') BOOTOPTIONS)

    /* Codepage, display, and keyboard layout */
    rc=configLine('COUNTRY='COUNTRY',\os2\country.sys')
    rc=configLine('CODEPAGE='CODEPAGE)
    rc=configLine('DEVINFO=KBD,'KEYBOARD',\os2\keyboard.dcp')
    rc=configLine('DEVINFO=SCR,VGA,\os2\viotbl.dcp')

    /* Unicode */
    if isSet(UNICODE) then do
      rc=configLine('SET LANG='UNILANG)
      rc=configLine('SET ULSPATH=\language')
      rc=configLine('DEVICE=\os2\'fileName(UNICODEFILE,'T'))
    end

    /* CDROM support */
    if isSet(CDROM) then rc=configLine('DEVICE=\os2\'fileName(CDROMDMDFILE,'T') CDROMOPTIONS)

    /* ASPI support */
    if isSet(ASPI)  then rc=configLine('BASEDEV='fileName(ASPIDMDFILE,'T') ASPIOPTIONS)

    /* Test device driver (required?) */
    rc=configLine('DEVICE='DRIVELETTER'\os2\testcfg.sys')

    /* Mouse support */
    if isSet(MOUSE) then do
        rc=configLine('DEVICE='DRIVELETTER'\os2\'fileName(MOUSEPOINTDDFILE,TRUNCATE))
        if MOUSEDEVFILE<>'' then
            rc=configLine('DEVICE='DRIVELETTER'\os2\'fileName(MOUSEDEVFILE,TRUNCATE) MOUSEDEVFILEOPTIONS)
        rc=configLine('DEVICE='DRIVELETTER'\os2\'fileName(MOUSEFILE,TRUNCATE) MOUSEFILEOPTIONS)
    end
    
    /* USB */
    if isSet(USB) then do
        do USBOHCI
            rc=configLine('BASEDEV='fileName(USBOHCIFILE,'T') USBOHCIOPTIONS)
        end
        do USBUHCI
            rc=configLine('BASEDEV='fileName(USBUHCIFILE,'T') USBUHCIOPTIONS)
        end
        do USBEHCI
            rc=configLine('BASEDEV='fileName(USBEHCIFILE,'T') USBEHCIOPTIONS)
        end
        rc=configLine('BASEDEV='fileName(USBDFILE,  'T') USBDOPTIONS)
        rc=configLine('BASEDEV='fileName(USBHIDFILE,'T') USBHIDOPTIONS)
        if isSet(USBKEYB)    then rc=configLine('DEVICE='DRIVELETTER'\os2\'fileName(USBKEYBFILE,   TRUNCATE) USBKEYBOPTIONS)
        if isSet(USBCOM)     then rc=configLine('DEVICE='DRIVELETTER'\os2\'fileName(USBCOMFILE,    TRUNCATE) USBCOMOPTIONS)
        if isSet(USBMOUSE)   then rc=configLine('DEVICE='DRIVELETTER'\os2\'fileName(USBMOUSEFILE,  TRUNCATE) USBMOUSEOPTIONS)
        if isSet(USBPRINTER) then rc=configLine('DEVICE='DRIVELETTER'\os2\'fileName(USBPRINTERFILE,TRUNCATE) USBPRINTEROPTIONS)
        if isSet(USBCDROM)   then rc=configLine('BASEDEV='fileName(USBCDROMFILE,'T')   USBCDROMOPTIONS)
        if isSet(USBSTORAGE) then rc=configLine('BASEDEV='fileName(USBSTORAGEFILE,'T') USBSTORAGEOPTIONS)
    end
    
    /* Serial port support */
    if isSet(SERIALPORT) then
        rc=configLine('DEVICE='DRIVELETTER'\os2\'fileName(SERIALPORTFILE,TRUNCATE) SERIALPORTOPTIONS)

    /* SIO Serial port support */
    if isSet(SIOSERIALPORT) then
        rc=configLine('DEVICE='DRIVELETTER'\os2\'fileName(SIOSERIALPORTFILE,TRUNCATE) SIOSERIALPORTOPTIONS)

    /* SIO2K Serial port support */
    if isSet(SIO2KSERIALPORT) then do
        if isSet(SIO2KPATCH) then
            rc=configLine('DEVICE='DRIVELETTER'\os2\'fileName(SIO2KPATCHFILE,TRUNCATE))
        rc=configLine('DEVICE='DRIVELETTER'\os2\'fileName(SIO2KUARTFILE,TRUNCATE) SIO2KUARTOPTIONS)
        rc=configLine('DEVICE='DRIVELETTER'\os2\'fileName(SIO2KSERIALPORTFILE,TRUNCATE) SIO2KSERIALPORTOPTIONS)
    end

    /* Parallel port support */
    if isSet(PARALLELPORT) then do
        rc=configLine('BASEDEV='filename(PARALLELPORTFILE,TRUNCATE) PARALLELPORTOPTIONS)
        rc=configLine('PRINTMONBUFSIZE=134,134,134')
    end

    /* APM */
    if isSet(APM) then do
        rc=configLine('DEVICE='DRIVELETTER'\os2\'filename(APMFILE,TRUNCATE) APMOPTIONS)
        if APMDAEMONFILE<>'' then rc=configLine('RUN='DRIVELETTER'\os2\'filename(APMDAEMONFILE,TRUNCATE) APMDAEMONOPTIONS)
    end

    /* WPS */
    if isSet(WPS) then do
        rc=configLine('DEVICE='DRIVELETTER'\os2\pmdd.sys')
        if isSet(CREATEBOOTCD) then do
            rc=configLine('CALL='filename(SHELLFILE,TRUNCATE) '/c mkdir' RAMDISKDRIVE'\OS2 > /dev/nul')
            rc=configLine('CALL='filename(SHELLFILE,TRUNCATE) '/c copy' CDROMDRIVE'\os2\*.ini' RAMDISKDRIVE'\OS2 > /dev/nul')
        end
        rc=configLine('SET USER_INI='ifSet(CREATEBOOTCD,RAMDISKDRIVE,CDROMDRIVE)'\os2\os2.ini')
        rc=configLine('SET SYSTEM_INI='ifSet(CREATEBOOTCD,RAMDISKDRIVE,CDROMDRIVE)'\os2\os2sys.ini')
        rc=configLine('SET AUTOSTART=PROGRAMS,TASKLIST,FOLDERS,WARPCENTER')
        rc=configLine('SET RESTARTOBJECTS=STARTUPFOLDERSONLY')
        if isSet(WPSAPPS) then rc=configLine('SET EPMPATH='CDROMDRIVE'\os2\apps')
    end

    /* Networking */
    if isSet(NETWORKING) then do
        rc=configLine('CALL='CDROMDRIVE'\network\netbind.exe')
        rc=configLine('DEVICE='CDROMDRIVE'\network\protman.os2  /I:'CDROMDRIVE'\network')

        /* TCP/IP */
        if isSet(TCPIP) then do
            rc=configLine('DEVICE='CDROMDRIVE'\network\sockets.sys')
            rc=configLine('DEVICE='CDROMDRIVE'\network\aflean.sys')
            rc=configLine('RUN='CDROMDRIVE'\network\bin\cntrl.exe')
            rc=configLine('SET HOSTNAME='TCPIPHOSTNAME)
            rc=configLine('SET USE_HOSTS_FIRST=1')
            ETCDIR=CDROMDRIVE'\network\etc'
            if isSet(CREATEBOOTCD) & isSet(RAMDISK) then do
                rc=configLine('CALL='filename(SHELLFILE,TRUNCATE) '/c mkdir' RAMDISKDRIVE'\ETC > /dev/nul')
                rc=configLine('CALL='filename(SHELLFILE,TRUNCATE) '/c xcopy' ETCDIR'\*' RAMDISKDRIVE'\ETC > /dev/nul')
                ETCDIR=RAMDISKDRIVE'\ETC'
            end
            rc=configLine('SET ETC='ETCDIR)
            rc=configLine('CALL='filename(SHELLFILE,TRUNCATE) '/c' CDROMDRIVE'\network\bin\tcpip.cmd > /dev/nul')
        end

        /* NETBIOS */
        if isSet(NETBIOS) then do
            rc=configLine('DEVICE='CDROMDRIVE'\network\netbeui.os2')
        end

        /* NETBIOS over TCP/IP */
        if isSet(NETBIOSTCPIP) then do
            rc=configLine('DEVICE='CDROMDRIVE'\network\tcpbeui.os2')
            rc=configLine('RUN='CDROMDRIVE'\network\nbtcp.exe')
        end

        /* NETBIOS or NETBIOS over TCP/IP */
        if isSet(NETBIOS) | isSet(NETBIOSTCPIP) then do
            rc=configLine('DEVICE='CDROMDRIVE'\network\netbios.os2')
        end

        /* Card driver */
        rc=configLine('DEVICE='CDROMDRIVE'\network\'fileName(NETWORKCARDFILE))
    end

    /* Extra device drivers */
    do I=1 to EXTRADEVICE
        rc=configLine('DEVICE='CDROMDRIVE'\os2\'fileName(EXTRADEVICEFILE.I,'T') EXTRADEVICEOPTIONS.I)
    end
    
    /* System settings */
    rc=configLine('MEMMAN='MEMMAN)
    rc=configLine('SWAPPATH='SWAPPATH)
    rc=configLine('IOPL=YES')
    rc=configLine('PROTECTONLY=YES')
    rc=configLine('AUTOFAIL=YES') /* Necessary for WPS CD boot! */
    rc=configLine('THREADS='THREADS)
    rc=configLine('DISKCACHE='FATCACHEOPTIONS)
    rc=configLine('MAXWAIT=1')

    /* Display settings */    
    if isSet(WPS) then rc=configLine('SET DISPLAYTYPE=VGA')
    select
        when isSet(WPS) & VIDEODRIVER=2 then do
            rc=configLine('SET VIDEO_DEVICES=VIO_SVGA')
            rc=configLine('SET VIO_SVGA=DEVICE(BVHVGA,BVHSVGA)')
            rc=configLine('SET C1=GENGRADD')
            rc=configLine('SET GRADD_CHAINS=C1')
            rc=configLine('DEVICE='CDROMDRIVE'\os2\gradd.sys')
        end
        otherwise do
            rc=configLine('SET VIDEO_DEVICES=VIO_VGA')
            rc=configLine('SET VIO_VGA=DEVICE(BVHVGA)')
        end
    end

    /* Command environment */
    TZ=value('TZ',,'OS2ENVIRONMENT')
    if TZ<>'' then rc=configLine('SET TZ='TZ)
    rc=configLine('SET PROMPT='SHELLPROMPT)
    rc=configLine('SET KEYS=ON')

    /* Shell */
    rc=configLine('PROTSHELL='ifSet(HPFS386,'A:\ibm386fs\bootsh.exe ')DRIVELETTER'\os2\'fileName(PROTSHELLFILE,TRUNCATE) PROTSHELLFILEOPTIONS)
    if isSet(WPS) then do
        rc=configLine('SET RUNWORKPLACE='DRIVELETTER'\os2\'fileName(WPSRUNWORKPLACEFILE,TRUNCATE) WPSRUNWORKPLACEFILEOPTIONS)
    end
    rc=configLine('SET OS2_SHELL='DRIVELETTER'\os2\'fileName(SHELLFILE,TRUNCATE) SHELLFILEOPTIONS)
    rc=configLine('SET COMSPEC='DRIVELETTER'\os2\'fileName(SHELLFILE,TRUNCATE))

    /* REXX */
    if isSet(REXX) & \ isSet(WPSBOOT) then
        rc=configLine('RUN='CDROMDRIVE'\os2\'filename(REXXCOMMANDLINEFILE))

    /* Additional config.sys lines */
    rc=configLine(ADDITIONALCFG)
    
    /* Close config.sys */
    rc=lineout(VIRTUALDRIVE'\config.sys')
    
    return rc


/* Create INI files on the CD --------------------------------------------- */
createINIFiles:
    call progress 1, 'Creating INI files'

    rc=parseINIRC(ifExists(WPSUSERINIRC, WPSUSERINIRC), TEMPDIR'\ini.rc')
    rc=parseINIRC(ifExists(WPSSYSINIRC,  WPSSYSINIRC),  TEMPDIR'\inisys.rc')

    /* Create the ini files based on the templates */
    'makeini' TEMPDIR'\os2.ini' TEMPDIR'\ini.rc' IGNOREOUTPUT
    if rc<>0 then
        call error 'Failed to create os2.ini file using template' WPSUSERINIRC
    'makeini' TEMPDIR'\os2sys.ini' TEMPDIR'\inisys.rc' IGNOREOUTPUT
    if rc<>0 then
        call error 'Failed to create os2sys.ini file using template' WPSSYSINIRC

    /* Add display resolution setting */
    if VIDEODRIVER<>1 and VIDEORESOLUTION<>0 then do
        /* Video resolution and color depth are encoded in 32bit */
        /* Little-Endian notation                                */
        select
            when VIDEORESOLUTION=2 then VIDEORESOLUTIONSTRING='20030000 58020000' /*  800x 600 */
            when VIDEORESOLUTION=3 then VIDEORESOLUTIONSTRING='00040000 00030000' /* 1024x 768 */
            when VIDEORESOLUTION=4 then VIDEORESOLUTIONSTRING='00050000 00040000' /* 1280x1024 */
            when VIDEORESOLUTION=5 then VIDEORESOLUTIONSTRING='40060000 b0040000' /* 1600x1200 */
            otherwise                   VIDEORESOLUTIONSTRING='80020000 e0010000' /*  640x 480 */
        end
        select
            when VIDEOCOLORDEPTH=2 then VIDEORESOLUTIONSTRING=VIDEORESOLUTIONSTRING '00800000' /* 32K colors */ 
            when VIDEOCOLORDEPTH=3 then VIDEORESOLUTIONSTRING=VIDEORESOLUTIONSTRING '00000100' /* 64K colors */
            when VIDEOCOLORDEPTH=4 then VIDEORESOLUTIONSTRING=VIDEORESOLUTIONSTRING '00000001' /* 16M colors */
            otherwise                   VIDEORESOLUTIONSTRING=VIDEORESOLUTIONSTRING '00010000' /* 256 colors */
        end
        /* Two extra 32bit integers don't know what they are for (refresh rate?) */
        VIDEORESOLUTIONSTRING=VIDEORESOLUTIONSTRING '01000000 40000000'
        rc=sysIni(TEMPDIR'\os2.ini', 'PM_DISPLAYDRIVERS', 'DEFAULTSYSTEMRESOLUTION', X2C(VIDEORESOLUTIONSTRING))
        if rc<>'' then
            call error 'Failed to add default system resolution to os2.ini' 'RC='rc
    end

    /* Move the ini files to CD */
    'copy' TEMPDIR'\os2.ini' WRITERDRIVE'\OS2' IGNOREOUTPUT
    if rc<>0 then ERR='Error copying' TEMPDIR'\os2.ini to' WRITERDRIVE'\OS2 rc('rc')'
    if rc=0 then do
        'del' TEMPDIR'\os2.ini' IGNOREOUTPUT
        if rc<>0 then ERR='Error deleting' TEMPDIR'\os2.ini rc('rc')'
    end
    if rc=0 then do
        'copy' TEMPDIR'\os2sys.ini' WRITERDRIVE'\OS2' IGNOREOUTPUT
        if rc<>0 then ERR='Error copying' TEMPDIR'\os2sys.ini to' WRITERDRIVE'\OS2 rc('rc')'
    end
    if rc=0 then do
        'del' TEMPDIR'\os2sys.ini' IGNOREOUTPUT
        if rc<>0 then ERR='Error deleting' TEMPDIR'\os2sys.ini rc('rc')'
    end
    if rc<>0 then
        call error(ERR)

    return 0



/* Parse INI file template ------------------------------------------------ */
parseINIRC:
    SOURCERC=ARG(1)
    DESTINATIONRC=ARG(2)

    call progress 2, 'Parsing INI file template' SOURCERC

    RCPARSE=4
    RCPARSE.1='$$'
    RCPARSE.2='%%'
    RCPARSE.3='$$'
    RCPARSE.4='%%'

    do while lines(SOURCERC)>0
        RCLINE=linein(SOURCERC)
        if left(strip(RCLINE),2)='/*' | left(strip(RCLINE),1)=';' then
            RCLINE=''
        do I=1 to RCPARSE
            STARTPOS=1
            do until STARTPOS=0
                STARTPOS=pos(RCPARSE.I, RCLINE, STARTPOS)
                if (STARTPOS<>0) then do
                    ENDPOS=pos(RCPARSE.I, RCLINE, STARTPOS+2)
                    if ENDPOS<>0 then do
                        LEFTRCLINE=left(RCLINE,STARTPOS-1)
                        RIGHTRCLINE=substr(RCLINE,ENDPOS+2)
                        MIDRCLINE=value(substr(RCLINE,STARTPOS+2,ENDPOS-STARTPOS-2))
                        if (I=2|I=4) & \ isSet(MIDRCLINE) then do
                            RCLINE=''
                            ENDPOS=0
                        end
                        else do
                            if I=2 then MIDRCLINE=''
                            RCLINE=LEFTRCLINE||MIDRCLINE||RIGHTRCLINE
                            ENDPOS=STARTPOS+length(MIDRCLINE)
                        end
                    end
                    STARTPOS=ENDPOS
                end
            end
        end
        rc=lineout(DESTINATIONRC, RCLINE)
    end

    /* Close files */
    rc=lineout(DESTINATIONRC)
    if rc<>0 then
        call error 'Failed to close file' DESTINATIONRC 'rc('rc')'
    rc=stream(SOURCERC, 'C', 'CLOSE')
    if rc<>'READY:' & rc<>'' then
        call error 'Failed to close file' SOURCERC 'rc('rc')'

    return 0


/* Create networking files on the CD -------------------------------------- */
createNetworkFiles:
    call progress 1, 'Creating network files'

    if isSet(TCPIP) then do
        /* tcpip.cmd */
        TXT=''||,
          'inetwait'||CRLF||,
          'route -fh'||CRLF||,
          'arp -f'||CRLF||,
          'ifconfig lo 127.0.0.1'||CRLF
        if isSet(TCPIPDHCP) then
            TXT=TXT||'dhcpstrt -i lan0'||CRLF
        else
            TXT=TXT||'ifconfig lan0' TCPIPHOSTADDRESS||CRLF
        if  strip(TCPIPGATEWAYADDRESS)<>'' then
            TXT=TXT||'route add default' TCPIPGATEWAYADDRESS||CRLF
        rc=createFile(WRITERDRIVE'\network\bin\tcpip.cmd',TXT)

        /* ETC Directory */
        rc=createDir(WRITERDRIVE'\NETWORK\ETC')

        /* dhcpcd.cfg */
        if isSet(TCPIPDHCP) then do
            TXT=''||,
              'clientid   MAC'||CRLF||,
              'option 1                     # Subnet Mask'||CRLF||,
              'option 3                     # Router'||CRLF||,
              'option 6                     # Domain Name Server'||CRLF||,
              'option 15                    # Domain Name'||CRLF||,
              'option 28                    # Broadcast Address'||CRLF||,
              'option 33                    # Static Routes'||CRLF||,
              'option 60 "IBMWARP_V4.1"     # Vendor Class'||CRLF||,
              'option 77 "IBMWARP_V4.1"     # User Class'||CRLF
            rc=createFile(WRITERDRIVE'\network\etc\dhcpcd.cfg',TXT)
        end

        /* protocol */
        TXT=''||,
          'icmp  1 ICMP'||CRLF||,
          'ip    4 IP'||CRLF||,
          'tcp   6 TCP'||CRLF||,
          'udp  17 UDP'||CRLF
        rc=createFile(WRITERDRIVE'\network\etc\protocol',TXT)

        /* services */
        TXT=''||,
          'ftp-data         20/tcp'||CRLF||,
          'ftp-data         20/udp'||CRLF||,
          'ftp              21/tcp'||CRLF||,
          'ftp              21/udp'||CRLF||,
          'telnet           23/tcp'||CRLF||,
          'telnet           23/udp'||CRLF||,
          'name             42/tcp'||CRLF||,
          'name             42/udp'||CRLF||,
          'nameserver       42/tcp'||CRLF||,
          'nameserver       42/udp'||CRLF||,
          'domain           53/tcp'||CRLF||,
          'domain           53/udp'||CRLF||,
          'tftp             69/tcp'||CRLF||,
          'tftp             69/udp'||CRLF||,
          'finger           79/tcp'||CRLF||,
          'finger           79/udp'||CRLF||,
          'hostname        101/tcp'||CRLF||,
          'hostname        101/udp'||CRLF||,
          'exec            512/tcp'||CRLF||,
          'login           513/tcp'||CRLF||,
          'shell           514/tcp'||CRLF
        rc=createFile(WRITERDRIVE'\network\etc\services',TXT)

        /* hosts */
        TXT=''||,
          '127.0.0.1 localhost'||CRLF||,
          ifClause(TCPIPHOSTADDRESS<>'' & \isSet(TCPIPDHCP), TCPIPHOSTADDRESS TCPIPHOSTNAME'.'TCPIPDOMAINNAME||CRLF)||,
          TCPIPHOSTS
        rc=createFile(WRITERDRIVE'\network\etc\hosts',TXT)

        /* resolv2 */
        if TCPIPDNSADDRESS<>'' then do
            TXT=''||,
              'domain' TCPIPDOMAINNAME||CRLF||,
              'nameserver' TCPIPDNSADDRESS||CRLF
            rc=createFile(WRITERDRIVE'\network\etc\resolv2',TXT)
        end
    end

    /* protocol.ini */
    NAME=translate(fileName(NETWORKCARDFILE))
    IDX=pos('.',NAME)
    if IDX>0 then NAME=left(NAME,IDX-1)
    TXT=''||,
      '[PROTMAN]'||CRLF||CRLF||,
      '  DRIVERNAME = PROTMAN$'||CRLF||CRLF||,
      '[IBMLXCFG]'||CRLF||CRLF||,
      ifSet(TCPIP,       '  tcpip_nif = tcpip.nif'||CRLF)||,
      ifSet(NETBIOS,     '  netbeui_nif = netbeui.nif'||CRLF)||,
      ifSet(NETBIOSTCPIP,'  tcpbeui_nif = tcpbeui.nif'||CRLF)||,
      '  NETWORK_nif = NETWORK.nif'||CRLF
    if isSet(TCPIP) then 
        TXT=TXT||CRLF||,
          '[tcpip_nif]'||CRLF||CRLF||,
          '  DriverName = TCPIP$'||CRLF||,
          '  Bindings = NETWORK_nif'||CRLF
    if isSet(NETBIOS) | isSet(NETBIOSTCPIP) then
        TXT=TXT||CRLF||,
          '[NETBIOS]'||CRLF||CRLF||,
          '  DriverName = netbios$'||CRLF
    ADAPTER=0
    if isSet(NETBIOS) then do
        TXT=TXT||'  ADAPTER'ADAPTER' = netbeui$,'ADAPTER||CRLF
        ADAPTER=ADAPTER+1
    end
    if isSet(NETBIOSTCPIP) then
        TXT=TXT||'  ADAPTER'ADAPTER' = tcpbeui$,'ADAPTER||CRLF
    if isSet(NETBIOS) then
        TXT=TXT||CRLF||,
          '[netbeui_nif]'||CRLF||CRLF||,
          '  DriverName = netbeui$'||CRLF||,
          '  Bindings = NETWORK_nif'||CRLF
    if isSet(NETBIOSTCPIP) then
        TXT=TXT||CRLF||,
          '[tcpbeui_nif]'||CRLF||,
          ''||CRLF||,
          '   DriverName = tcpbeui$'||CRLF||,
          '   Bindings = NETWORK_nif'||CRLF||,
          '   NODETYPE = "B-Node"'||CRLF
    TXT=TXT||CRLF||,
      '[NETWORK_nif]'||CRLF||CRLF||,
      '  DriverName = 'NAME'$'||CRLF
    rc=createFile(WRITERDRIVE'\network\protocol.ini',TXT)
    
    return 0


/* Create extra files on the CD ------------------------------------------- */
createExtraCDFiles:
    if EXTRACREATEFILES>0 then call progress 1, 'Creating extra files'

    do I=1 to EXTRACREATEFILES
        rc=createDir(WRITERDRIVE||EXTRACREATEFILEDIRECTORY.I)
        rc=createFile(WRITERDRIVE||EXTRACREATEFILEDIRECTORY.I||'\'||EXTRACREATEFILENAME.I,EXTRACREATEFILECONTENT.I)
    end

    return rc
